#! perl

=head1 NAME

RSSCache::Admin::Console - Administration interface on the console

=head1 SYNOPSIS

	my $console = RSSCache::Admin::Console->new();

=head1 DESCRIPTION

Offers a simple administration interface on the console/shell. Will start a new
thread, which waits for user input.

Type 'help' to get a brief description of the available commands.

=cut

package RSSCache::Admin::Console;

use strict;
use warnings;

use threads;
use Term::ReadLine;

=head1 METHODS

=head2 new

Constructor.

=cut
sub new {
	my ($class) = @_;

	#create object
	my $self = bless {}, $class;
	
	#available commands
	$self->{commands} = {
		help => {
			command => 'help',
			description => 'Prints out all available commands and their description.',
			code => \&help
		},
		quit => {
			command => 'quit',
			description => 'Exits this program.',
			code => \&quit
		},
		feeds => {
			command => 'feeds',
			description => 'Prints out a list of all feeds that are currently available.',
			code => \&feeds
		},
		feed => {
			command => 'feed url client_id',
			description => 'Prints out the current cache of a feed with the specified url (optional: for the specified client id).',
			code => \&feed
		},
		set => {
			command => 'set [key [value]]',
			description => "Sets the setting to the specified value.\nIf no value is defined, the current value of the setting will be printed.\nIf also no key is defined, all settings will be printed.\nNote that most settings will only be evaluated at startup!",
			code => \&set
		},
		poll => {
			command => 'poll [url]',
			description => "Will read the specified feed. If no url is specified, all feeds will be read.\nIf the feed hasn't been cached yet, it will be added to the cache.",
			code => \&poll
		},
		delete => {
			command => 'delete [url]',
			description => "Deletes the feed with the specified url. If no url is specified, all feeds will be deleted.",
			code => \&delete
		},
		uptime => {
			command => 'uptime',
			description => "Shows how long this application is running.",
			code => \&uptime
		},
	};
	
	#create, start and return thread
	return threads->new(\&start, $self);
}
# /new

#will start the console thread
sub start {
	my ($self) = @_;
	
	$RSSCache::Log->info(__PACKAGE__." started. Type 'help' for a brief information about the available commands.");
	
	#use readline to support command history etc.
	my $term = Term::ReadLine->new('RSSCache');
	my $prompt = "$0> ";
	#Term::ReadLine doesn't work on my win32 machine
	if ($^O =~ /win32/i) {
		print $prompt;
		while (my $line = <>) {
			chomp $line;
			next unless $line;
			my ($command, @params) = split /\s+/, $line;
			if (exists $self->{commands}->{$command}) {
				&{$self->{commands}->{$command}->{code}}($self, @params);
			} else {
				print "Invalid command: '$line'. Type 'help' to get a list of available commands.\n";
			}
			print $prompt;
		}
	} else {
		while (defined (my $line = $term->readline($prompt))) {
			next unless $line;
			my ($command, @params) = split /\s+/, $line;
			if (exists $self->{commands}->{$command}) {
				&{$self->{commands}->{$command}->{code}}($self, @params);
			} else {
				print "Invalid command: '$line'. Type 'help' to get a list of available commands.\n";
			}
			$term->addhistory($line) if $line =~ /\S/;
		}
	}
}
# /start

#will put out a brief description of the available commands.
sub help {
	my ($self) = @_;
	
	#find longest command
	my $len = 0;
	foreach my $key (keys %{$self->{commands}}) {
		$len = length $self->{commands}->{$key}->{command} if length $self->{commands}->{$key}->{command} > $len;
	}
	#print out help screen
	foreach my $key (sort keys %{$self->{commands}}) {
		my @lines = split /\n/, $self->{commands}->{$key}->{description};
		print $self->{commands}->{$key}->{command} . ' ' x ($len - length $self->{commands}->{$key}->{command}) . " - " . (shift @lines) . "\n";
		foreach my $line (@lines) {
			print ' ' x $len . "   $line\n";
		}
	}
}
# /help

#exits the program.
sub quit {
	my ($self) = @_;
	RSSCache::stop();
}
# /quit

#prints out a list of the currently cached feeds.
sub feeds {
	my ($self) = @_;
	
	my @feeds = sort { $a cmp $b } $RSSCache::Backend->feeds();
	my $items = 0;
	foreach my $url (@feeds) {
		my $count = $RSSCache::Backend->item_count($url);
		$items += $count;
		print "$url ($count item" . ($count == 1 ? '' : 's') . ")\n";
	}
	print scalar(@feeds) . " feed" . (@feeds == 1 ? '' : 's') . " loaded ($items item" . ($items == 1 ? '' : 's') . ").\n";
}
# /feeds

#prints out the URL and the current cache of a specified feed.
#if the feed hasn't been cached yet, it will be added to the cache.
sub feed {
	my ($self, $url, $cid) = @_;
	
	my $feed = $RSSCache::Backend->feed($url, $cid);
	print "Current cache for $url\n\n$feed\n\n" if defined $feed;
}

#Sets the setting to the specified value.
#If no value is defined, the current value of the setting will be printed.
#If also no key is defined, all settings will be printed.
sub set {
	my ($self, $key, $value) = @_;
	
	if (defined $key) {
		if (defined $value) {
			#set setting
			print "Setting '$key' to '$value'\n";
			$RSSCache::Settings->set($key, $value);
		} else {
			#print out setting
			$value = $RSSCache::Settings->get($key, $value);
			if (defined $value) {
				print "'$key' set to '$value'\n";
			} else {
				print "'$key' currently not set\n";
			}
		}
	} else {
		#show all settings
		
		#get keys
		my @keys = sort $RSSCache::Settings->keys();
		
		#find longest key
		my $len = 0;
		foreach my $key (@keys) {
			$len = length $key if length $key > $len;
		}
		
		#print out settings
		foreach my $key (@keys) {
			print $key . ' ' x ($len - length $key) . " - " . (defined $RSSCache::Settings->get($key) ? $RSSCache::Settings->get($key) : '') . "\n";
		}
	}
}

#polls the specified feeds or all cached feeds, if no feeds have been specified.
sub poll {
	my ($self, @feeds) = @_;
	
	my $result = $RSSCache::Poller->poll(@feeds);
	$self->feed($feeds[0]) if @feeds == 1;
	return $result;
}

#deletes a specified feed
sub delete {
	my ($self, $url) = @_;
	
	if ($url) {
		$RSSCache::Backend->delete_feed($url);
	} else {
		$RSSCache::Backend->clear();
	}
}

#shows how long this program is running
sub uptime {
	my ($self) = @_;
	
	my $diff = time() - $RSSCache::StartTime;
	my $secs  = $diff % 60; $diff = int($diff / 60);
	my $mins  = $diff % 60; $diff = int($diff / 60);
	my $hours = $diff % 24; $diff = int($diff / 24);
	my $days  = $diff;
	print "Running since " . $RSSCache::Log->format_date($RSSCache::StartTime) . ". Uptime: $days day" . ($days == 1 ? '' : 's') . ", $hours hour" . ($hours == 1 ? '' : 's') . ", $mins minute" . ($mins == 1 ? '' : 's') . ", $secs second" . ($secs == 1 ? '' : 's') . "\n";
}

=head1 AUTHOR

Copyright 2006 Thomas Wittek (mail at gedankenkonstrukt dot de). All rights reserved. 

=head1 LICENsE

This document is free software.
It is distributed under the same terms as Perl itself.

=head1 SEE ALSO

L<RSSCache>

=cut

1;
