#! perl

#TODO: package-scope %store as workaround for missing reference support in threads::shared isn't beautiful.
#      all complex data structures have to be flattened before storing them in the shared variable.

=head1 NAME

RSSCache::Backend::Memory - Backend implementations for storage in the memory (RAM).

=head1 SYNOPSIS

	use RSSCache::Backend::Memory;
	$RSSCache::Backend = RSSCache::Backend::Memory->new();

=head1 DESCRIPTION

This class is a L<RSSCache::Backend> implementation which stores all the feeds
inside the memory (RAM, i.e. in an hash).

=cut

package RSSCache::Backend::Memory;

use strict;
use warnings;

use threads::shared;
use Storable qw/freeze thaw store retrieve/; #complex data will be freezed before saving them in the shared variable

#this class is a backend
use RSSCache::Backend;
@RSSCache::Backend::Memory::ISA = qw/RSSCache::Backend/;

my %store : shared;

=head1 METHODS

Since all the methods of this class have been documented in L<RSSCache::Backend>
they won't be documented here once again.

=cut

sub new {
	my ($class) = @_;
	
	#create myself, inherit base class constuctor
	my $self = RSSCache::Backend->new();
	
	#try to retrieve already stored data
	my $stored;
	eval {
		my $filename = $RSSCache::Settings->get('backend/data_file');
		$stored = retrieve $filename;
		my $feeds = scalar map { $_ =~ /^cid_/ ? () : $_ } keys %{$stored};
		my $cids  = scalar(keys %{$stored}) - $feeds;
		$RSSCache::Log->notice(__PACKAGE__."->new(): Loaded $feeds feed" . ($feeds == 1 ? '' : 's') . " and $cids cid" . ($cids == 1 ? '' : 's') . " from $filename.");
	};
	%store = (defined $stored ? %{$stored} : ());
	
	#rebless myself
	return bless $self, $class;
}

sub feeds {
	my ($self) = @_;
	return map { $_ =~ /^cid_/ ? () : $_ } keys %store;
}

sub feed_loaded {
	my ($self, $url) = @_;
	return exists $store{$url};
}

sub feed {
	my ($self, $url, $cid) = @_;
	
	$url = "http://$url" unless $url =~ /^https?:\/\//i;
	#retrieve feed
	my $feed = $self->_get_feed($url);
	#load feed if it doesn't exist
	unless (defined $feed) {
		$RSSCache::Log->info(__PACKAGE__."->feed(): Feed '$url' hasn't been cached yet. Adding it.");
		if ($RSSCache::Poller->poll_feed($url)) {
			$feed = $self->_get_feed($url);
		} else {
			return undef;
		}
	}
	#serve feed
	if (defined $feed) {
		if (defined $cid) {
			#create client id unless it exists
			$self->create_client_id($cid) unless $self->client_id_exists($cid);
			#the time this client polled this feed the last time + a "bonus" of 60 seconds to avoid missing feeds
			my $last = defined $feed->{client_polls}->{$cid} ? $feed->{client_polls}->{$cid} - 60 : 0;
			#render feed individually for this client
			#filter and sort by date
			my @items = map { $feed->{items}->{$_} > $last ? $_ : () } keys %{$feed->{items}};
			my $rendered = '';
			if (@items) {
				@items = sort { $feed->{items}->{$b} <=> $feed->{items}->{$a} } @items;
				$rendered = $feed->{start} . (join "\n", @items) . $feed->{end};
			}
			#update last polling time
			$feed->{client_polls}->{$cid} = time();
			$self->_store_feed($url, $feed);
			#return individual feed
			return $rendered;
		} elsif (defined $feed and $feed->{rendered}) {
			#serve complete feed
			return $feed->{rendered};
		}
	} else {
		return undef;
	}
}

sub item_count {
	my ($self, $url) = @_;
	
	$url = "http://$url" unless $url =~ /^https?:\/\//i;
	#retrieve feed
	my $feed = $self->_get_feed($url);
	if (defined $feed) {
		return scalar keys %{$feed->{items}};
	} else {
		$RSSCache::Log->info(__PACKAGE__."->feed(): Feed '$url' hasn't been cached so we've got no items yet.");
		return 0;
	}
}

sub feed_error {
	my ($self, $url) = @_;
	
	$url = "http://$url" unless $url =~ /^https?:\/\//i;
	#retrieve feed
	my $feed = $self->_get_feed($url);
	if (defined $feed) {
		#delete feed if the last successful download is longer ago than allowed
		$self->delete_feed($url) if $feed->{last_download} < time() - $self->configtime_to_unixtime($RSSCache::Settings->get('backend/max_offtime'));
	}
}

#parse data, update head, add new items, compile feed
sub add_or_update_feed {
	my ($self, $url, $data) = @_;
	
	$url = "http://$url" unless $url =~ /^https?:\/\//i;
	my $new_feed = $self->parse_feed($url, $data);
	if (defined $new_feed) {
		#create entry unless exists
		my $feed = $self->_get_feed($url);
		unless (defined $feed) {
			$RSSCache::Log->info(__PACKAGE__."->add_or_update_feed(): Adding new feed $url.");
			$feed = {};
		}
		
		#update start, end and date
		$feed->{start}         = $new_feed->{start};
		$feed->{end}           = $new_feed->{end};
		$feed->{last_download} = time();
		
		#add new items
		#the items will be stored in the hash for quick access:
		#$feed->{items}->{$item_text} = <date of addition>;
		#the date of addition will be decremented by one second for each item 
		#to keep the order of the incoming items.
		my $now = time();
		my $added = 0;
		foreach my $item (@{$new_feed->{items}}) {
			#only add new feeds
			unless (exists $feed->{items}->{$item}) {
				$feed->{items}->{$item} = $now--;
				$added++;
			}
		}
		$RSSCache::Log->notice(__PACKAGE__."->add_or_update_feed(): Added $added new item" . ($added == 1 ? '' : 's') . " for feed '$url'.") if $added;
		
		#delete outdated items
		my $deleted = 0;
		my $cache_deadline = time() - $self->configtime_to_unixtime($RSSCache::Settings->get('backend/cache_duration'));
		foreach my $item (keys %{$feed->{items}}) {
			if ($feed->{items}->{$item} < $cache_deadline) {
				delete $feed->{items}->{$item};
				$deleted++;
			}
		}
		$RSSCache::Log->notice(__PACKAGE__."->add_or_update_feed(): Deleted $deleted outdated item" . ($deleted == 1 ? '' : 's') . " for feed '$url'.") if $deleted;
		
		#"render" whole feed into a string to return it quickly on request
		my @items = sort { $feed->{items}->{$b} <=> $feed->{items}->{$a} } keys %{$feed->{items}};
		$feed->{rendered} = $feed->{start} . (join "\n", @items) . $feed->{end};
		
		#update store
		$self->_store_feed($url, $feed);
		
		return 1;
	} else {
		return undef;
	}
}

sub delete_feed {
	my ($self, $url) = @_;
	
	$url = "http://$url" unless $url =~ /^https?:\/\//i;
	if (exists $store{$url}) {
		lock %store;
		delete $store{$url};
		$RSSCache::Log->info(__PACKAGE__."->delete_feed(): Feed '$url' deleted.");
	} else {
		$RSSCache::Log->warning(__PACKAGE__."->delete_feed(): Feed '$url' doesn't exist.");
	}
	return 1;
}

sub clear {
	my ($self) = @_;
	
	lock %store;
	%store = ();
	$RSSCache::Log->info(__PACKAGE__."->clear(): Whole cache deleted.");
	return 1;
}

sub _get_feed {
	my ($self, $url) = @_;
	
	$url = "http://$url" unless $url =~ /^https?:\/\//i;
	my $feed;
	if (exists $store{$url} and defined $store{$url}) {
		$feed = thaw $store{$url};
	}
	return $feed;
}

sub _store_feed {
	my ($self, $url, $feed) = @_;
	
	$url = "http://$url" unless $url =~ /^https?:\/\//i;
	lock %store;
	$store{$url} = freeze $feed;
	$RSSCache::Log->debug(__PACKAGE__."->_store_feed(): Feed $url stored.");
	return 1;
}

sub client_id_exists {
	my ($self, $cid) = @_;
	return exists $store{"cid_$cid"};
}

sub create_client_id {
	my ($self, $cid) = @_;
	
	lock %store;
	$store{"cid_$cid"} = 1;
	$RSSCache::Log->debug(__PACKAGE__."->create_client_id(): CID $cid created.");
	return 1;
}

sub store_data {
	my ($self, $cid) = @_;
	
	my $filename = $RSSCache::Settings->get('backend/data_file');
	lock %store;
	my $feeds = scalar map { $_ =~ /^cid_/ ? () : $_ } keys %store;
	my $cids  = scalar(keys %store) - $feeds;
	store { %store }, $filename;
	$RSSCache::Log->notice(__PACKAGE__."->store_data(): Stored $feeds feed" . ($feeds == 1 ? '' : 's') . " and $cids cid" . ($cids == 1 ? '' : 's') . " to $filename.");
	return 1;
}

=head1 AUTHOR

Copyright 2006 Thomas Wittek (mail at gedankenkonstrukt dot de). All rights reserved. 

=head1 LICENsE

This document is free software.
It is distributed under the same terms as Perl itself.

=head1 SEE ALSO

L<RSSCache>, L<RSSCache::Backend>

=cut

1;
