#! perl

=head1 NAME

RSSCache::Log - Logging interface

=head1 SYNOPSIS

	#usage on startup
	$RSSCache::Log = RSSCache::Log->new();
	
	#usage within your modules
	$RSSCache::Log->debug  (__PACKAGE__."->your_method(): Your message.");
	$RSSCache::Log->notice (__PACKAGE__."->your_method(): Your message.");
	$RSSCache::Log->info   (__PACKAGE__."->your_method(): Your message.");
	$RSSCache::Log->warning(__PACKAGE__."->your_method(): Your message.");
	$RSSCache::Log->error  (__PACKAGE__."->your_method(): Your message.");

=head1 DESCRIPTION

This module will offer simple interface to logging of some messages the user
might be interested in.

The messages may be colored according to their type and they may be suppressed
according to the specified settings.

=cut

package RSSCache::Log;

use strict;
use warnings;

#colors don't really work on the windows console
use constant COLORED_OUTPUT => $^O !~ /win/i;
#use constant COLORED_OUTPUT => 0;

if (COLORED_OUTPUT) {
	use Term::ANSIColor qw/:constants/;
}

=head1 METHODS

=head2 new

Constructor.

=cut
sub new {
	my ($class, $settings) = @_;
	
	#set default settings
	$RSSCache::Settings->set('log/error',   1) unless defined $RSSCache::Settings->get('log/error');
	$RSSCache::Settings->set('log/warning', 1) unless defined $RSSCache::Settings->get('log/warning');
	$RSSCache::Settings->set('log/info',    1) unless defined $RSSCache::Settings->get('log/info');
	$RSSCache::Settings->set('log/notice',  0) unless defined $RSSCache::Settings->get('log/notice');
	$RSSCache::Settings->set('log/debug',   0) unless defined $RSSCache::Settings->get('log/debug');
	
	#return object
	return bless {}, $class;
}

=head2 error

Puts an error message.

=head3 Parameters:

=over

=item * $message - Your message.

=back

=cut
sub error {
	my ($self, $message) = @_;
	return unless $RSSCache::Settings->get('log/error');
	if (COLORED_OUTPUT) {
		print STDERR RED $self->_now(), "Error:  ", RESET, "\b\b $message\n";
	} else {
		print STDERR $self->_now(), " Error:  $message\n";
	}
}
# /error

=head2 warning

Puts a warning.

=head3 Parameters:

=over

=item * $message - Your message.

=back

=cut
sub warning {
	my ($self, $message) = @_;
	return unless $RSSCache::Settings->get('log/warning');
	if (COLORED_OUTPUT) {
		print STDERR YELLOW $self->_now(), "Warning:", RESET, "\b\b $message\n";
	} else {
		print STDERR $self->_now(), " Warning: $message\n";
	}
}
# /warning

=head2 info

Puts an info message.

=head3 Parameters:

=over

=item * $message - Your message.

=back

=cut
sub info {
	my ($self, $message) = @_;
	return unless $RSSCache::Settings->get('log/info');
	if (COLORED_OUTPUT) {
		print STDERR GREEN $self->_now(), "Info:   ", RESET, "\b\b $message\n";
	} else {
		print STDERR $self->_now(), " Info:    $message\n";
	}
}
# /info

=head2 notice

Puts a notice.

=head3 Parameters:

=over

=item * $message - Your message.

=back

=cut
sub notice {
	my ($self, $message) = @_;
	return unless $RSSCache::Settings->get('log/notice');
	if (COLORED_OUTPUT) {
		print STDERR BLUE $self->_now(), "Notice: ", RESET, "\b\b $message\n";
	} else {
		print STDERR $self->_now(), " Notice:  $message\n";
	}
}
# /notice

=head2 debug

Puts an debug message.

=head3 Parameters:

=over

=item * $message - Your message.

=back

=cut
sub debug {
	my ($self, $message) = @_;
	return unless $RSSCache::Settings->get('log/debug');
	if (COLORED_OUTPUT) {
		print STDERR BLUE $self->_now(), "Debug:  ", RESET, "\b\b $message\n";
	} else {
		print STDERR $self->_now(), " Debug:   $message\n";
	}
}
# /debug

#formats the given date and time
sub format_date {
	my ($self, $time) = @_;
	
	#generate current date formatted as YYYY-MM-DD-HH-MM
	my (undef, $min, $hour, $day, $mon, $year) = localtime $time;
	$mon++; $year += 1900;
	return sprintf '%04d-%02d-%02d %02d:%02d', $year, $mon, $day, $hour, $min;
}
# /format_date

#formats the current date and time to put it out with each log message
sub _now {
	my ($self) = @_;
	return $self->format_date(time());
}
# /_now

=head1 AUTHOR

Copyright 2006 Thomas Wittek (mail at gedankenkonstrukt dot de). All rights reserved. 

=head1 LICENsE

This document is free software.
It is distributed under the same terms as Perl itself.

=head1 SEE ALSO

L<RSSCache>

=cut

1;
