#! perl

#TODO: package-scope %store as workaround for missing reference support in threads::shared isn't beautiful

=head1 NAME

RSSCache::Settings - Module to read and access the settings

=head1 SYNOPSIS

	#use at startup
	$RSSCache::Settings = RSSCache::Settings->new();
	
	#use within your modules
	$RSSCache::Settings->set('my/setting', 'some value');
	my $value = $RSSCache::Settings->get('my/setting');

=head1 DESCRIPTION

Will read the RSSCache.settings file at startup and offers access to these settings.

=cut

package RSSCache::Settings;

use strict;
use warnings;

use threads::shared;

my %store : shared;

=head1 METHODS

=head2 new

Constructor.

Will initialize the settings store and load the settings.

=head3 Parameters:

=over

=item * $file - The file which should be loaded to initialize the settings store

=back

=cut
sub new {
	my ($class, $file) = @_;
	my $self = bless {}, $class;
	%store = ();
	$self->load_settings($file);
	return $self;
}
# /new

=head2 load_settings

Will initialize the settings store and load the settings from the
specified file.

If no file is specified, this method will try to load the feil RSSCache.settings.

=head3 Parameters:

=over

=item * $file - The file which should be loaded to initialize the settings store

=back

=cut
sub load_settings {
	my ($self, $filename) = @_;
	
	$filename ||= 'RSSCache.settings';
	my $file;
	if (-e $filename and -f $filename) {
		unless (open FILE, $filename) {
			die __PACKAGE__."->load_settings(): Could not open '$filename'! $!";
			return undef;
		}
		local $/ = undef;
		$file = <FILE>;
		close FILE;
	} else {
		die __PACKAGE__."->load_settings(): File '$filename' doesn't exist!";
		return undef;
	}
	
	my @lines = split /(\r?\n|\r)/, $file;
	my ($setting,$value);
	foreach (@lines) {
		if ($_) {
			if (/\#/) {#We have an inline comment. Kill everything behind it
				$_ =~ s/\#.*//o;
			}
			$_ =~ s/\s*$//o;#Kill tailing spaces
			($setting,$value) = (split /\s+/, $_, 2);
			#no undefs please
			$value = "" if not defined $value;
			$self->set($setting => $value) if defined $setting;
		}#if $_
	}#foreach @lines
	
	return 1;
}
#= /load_settings

=head2 get

Returns a value from the settings.

=head3 Parameters:

=over

=item * $key - The settings key

=back

=cut
sub get {
	my ($self, $key) = @_;
	
	#return (exists $self->{_reg}->{$key} ? $self->{_reg}->{$key} : undef);
	return (exists $store{$key} ? $store{$key} : undef);
}
#= /get

=head2 set

Works similary to get() but accepts two parameters
and lets you set values instead of reading them.

=head3 Parameters:

=over

=item * $key - The name of the entry

=item * $value - The value you want to set

=back

=cut
sub set {
	my ($self, $key, $value) = @_;
	
	#$self->{_reg}->{$key} = $value;
	lock %store;
	$store{$key} = $value;
}
#= /set

=head2 delete

Works similary to get() and set() but instead of reading or writing from or to
an entry you simply clear a value of a specific entry.

=head3 Parameters:

=over

=item * $key - The entry to delete

=back

=cut
sub delete {
	my ($self, $key) = @_;

	#delete $self->{_reg}->{$key};
	lock %store;
	delete $store{$key};
}
#= /delete

=head2 keys

Returns an array containing all settings keys.

=cut
sub keys {
	my ($self) = @_;

	#return keys %{$self->{_reg}};
	return keys %store;
}
#= /keys

=head1 AUTHOR

Copyright 2006 Thomas Wittek (mail at gedankenkonstrukt dot de). All rights reserved. 

=head1 LICENsE

This document is free software.
It is distributed under the same terms as Perl itself.

=head1 SEE ALSO

L<RSSCache>

=cut

1;
